<h3>{{ _('Formats') }}</h3>
<ul class="this-page-menu">
  {%- if show_source and has_source and sourcename %}
  <li><a href="{{ pathto('_sources/' + sourcename, true)|e }}" rel="nofollow">{{ _("This Page's Source") }}</a></li>
  {%- endif %}
  <li><a href="/{{ theme_manual_path }}/single/">{{ _('MongoDB Manual, Single HTML Page') }}</a></li>
  <li><a href="{{ theme_pdfpath }}" rel="nofollow">{{ _('MongoDB Manual, PDF Format') }}</a></li>
  <li><a href="{{ theme_epubpath }}" rel="nofollow">{{ _('MongoDB Manual, ePub Format') }}</a></li>
</ul>
