{%- if theme_project != 'manual' %}
<h3 style="padding-bottom: 1em;"><a href="http://docs.mongodb.org/manual/">MongoDB Manual</a></h3>
{%- endif %}

{%- if theme_project != 'ecosystem' %}
<h3 style="padding-bottom: 1em;"><a href="http://docs.mongodb.org/ecosystem/">MongoDB Ecosystem</a></h3>
{%- endif %}

{%- if theme_project != 'about' %}
<h3 style="padding-bottom: 1em;"><a href="http://www.mongodb.org/about/">About MongoDB</a></h3>
{%- endif %}
